package edu.travel.tenant.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import edu.travel.service.SysServiceImpl;
import edu.travel.tenant.dto.AssignRolesOrMenus;
import edu.travel.tenant.entity.EduTenant;
import edu.travel.tenant.entity.SysMenu;
import edu.travel.tenant.mapper.SysMenuMapper;
import edu.travel.tenant.service.*;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;

/**
 * ISysRoleServiceImpl 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/2/6
 */

@Service
public class ISysMenuServiceImpl extends SysServiceImpl<SysMenuMapper, SysMenu> implements ISysMenuService {

    @Autowired
    private ISysUserRoleService sysUserRoleService;
    @Autowired
    private ISysRoleMenuService sysRoleMenuService;
    @Autowired
    private ISysMenuService sysMenuService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ITenantService tenantService;

    @Override
    public void addMenu(SysMenu sysMenu) {
        //参数校验
        Assert.isTrue(ObjectUtil.isNotEmpty(sysMenu),"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(sysMenu.getPath()),"页面路由不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(sysMenu.getComponent()),"组件路径不能为空");
        //保存菜单
        super.save(sysMenu);
    }

    @Override
    public void deleteMenuByIds(List<Long> ids) {
        //参数校验
        Assert.isTrue(ObjectUtil.isNotEmpty(ids),"请选择想要删除的菜单");
        List<SysMenu> sysMenus = super.listByIds(ids);
        Assert.isTrue(ObjectUtil.isNotEmpty(sysMenus),"选择的菜单不存在或已删除");
        Assert.isTrue(sysMenus.size() == ids.size(),"包含重复的菜单或已删除菜单");
        sysMenus.forEach(sysMenu -> {
            sysMenu.setDeleteFlag(1);
        });
        this.updateBatchById(sysMenus);
    }


}
