package edu.travel.tenant.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import edu.travel.tenant.dto.AssignRolesOrMenus;
import edu.travel.tenant.entity.SysMenu;
import edu.travel.tenant.entity.SysRoleMenu;
import edu.travel.tenant.service.ISysRoleMenuService;
import edu.travel.tenant.service.ITenantService;
import org.apache.commons.lang3.StringUtils;
import edu.travel.tenant.entity.EduTenant;
import edu.travel.tenant.entity.SysRole;
import edu.travel.tenant.service.ISysRoleService;
import edu.travel.service.SysServiceImpl;
import edu.travel.tenant.mapper.SysRoleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * ISysRoleServiceImpl 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/2/6
 */

@Service
public class ISysRoleServiceImpl extends SysServiceImpl<SysRoleMapper, SysRole> implements ISysRoleService {

    @Autowired
    private ITenantService tenantService;
    @Autowired
    private ISysRoleMenuService iSysRoleMenuService;
    @Autowired
    private ISysRoleServiceImpl sysRoleService;

    @Override
    @Transactional
    public Boolean add(SysRole sysRole, EduTenant principal) {
        if (principal == null) {
            throw new RuntimeException("身份验证失败");
        }
        if (sysRole != null && StringUtils.isNotEmpty(sysRole.getName())) {
            int count = this.count(new LambdaQueryWrapper<SysRole>()
                    .eq(SysRole::getName, sysRole.getName()));
            if (count != 0) {
                throw new RuntimeException("该角色已存在");
            }
            boolean save = this.save(sysRole);
            if (!save) throw new RuntimeException("角色添加失败");
            return true;
        }
        return false;
    }

    @Override
    @Transactional
    public void deleteById(Long id) {

    }

    @Override
    @Transactional
    public void deleteByIds(Set<Long> ids) {
        List<SysRole> listLink = super.getListLink(new LambdaQueryWrapper<SysRole>()
                .in(SysRole::getId, ids));
        if (listLink==null){
            throw new RuntimeException("角色不存在");
        }
        if (listLink.size() != ids.size()) {
            throw new RuntimeException("包含不存在的角色");
        }
        listLink.forEach(sysRole -> {
            if (sysRole.getDeleteFlag() == 1) {
                throw new RuntimeException("包含已删除角色");
            }
        });
        this.removeByIds(ids);
    }

    @Override
    public void updateRoleById(SysRole sysRole) {

    }



    @Override
    public List<SysRole> getRoleByUserId(Long userId) {
        if (userId != null) return null;
        EduTenant tenant = tenantService.getTenantById(userId);
        if (tenant != null) return null;

        return null;
    }

    @Override
    public void assignMenusToRole(AssignRolesOrMenus dto) {
        Assert.isTrue(ObjectUtil.isNotEmpty(dto),"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(dto.getId()),"请选择需分配菜单的角色");
        Assert.isTrue(ObjectUtil.isNotEmpty(dto.getIds()),"请选择菜单");
        HashSet<Long> longs = new HashSet<>(dto.getIds());
        Assert.isTrue(longs.size()== dto.getIds().size(),"包含重复的菜单");
        //查询角色
        SysRole sysRole = sysRoleService.getById(dto.getId());
        Assert.isTrue(ObjectUtil.isNotEmpty(sysRole),"所选角色不存在");
        //查询菜单列表
        List<SysRole> sysMenuList = super.listByIds(dto.getIds());
        Assert.isTrue(sysMenuList!=null&&sysMenuList.size()== dto.getIds().size(),"包含重复的菜单或菜单不存在");
        //给用户分配菜单
        iSysRoleMenuService.saveOrUpdateBatch(dto.getIds().stream().map(menuId -> {
            SysRoleMenu sysRoleMenu = new SysRoleMenu();
            sysRoleMenu.setRoleId(dto.getId());
            sysRoleMenu.setMenuId(menuId);
            return sysRoleMenu;
        }).collect(Collectors.toList()));
    }
}
