package edu.travel.tenant.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import edu.travel.tenant.dto.AssignRolesOrMenus;
import edu.travel.tenant.entity.*;
import edu.travel.tenant.enums.UserStatus;
import edu.travel.tenant.service.*;
import edu.travel.service.SysServiceImpl;
import edu.travel.tenant.mapper.EduTenantMapper;
import edu.travel.tenant.vo.EduTenantVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ITenantServiceImpl extends SysServiceImpl<EduTenantMapper, EduTenant> implements ITenantService {

    @Autowired
    private ISysUserRoleService sysUserRoleService;
    @Autowired
    private ISysRoleMenuService sysRoleMenuService;
    @Autowired
    private ISysMenuService sysMenuService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Override
    public EduTenant getTenantByPhoneNumber(String tenantID) {
        return super.getOneLink(new QueryWrapper<EduTenant>().eq("tenant_phone",tenantID));
    }

    @Override
    @Transactional
    public void addTenant(EduTenant tenant) {
        if (tenant == null) {
            throw new RuntimeException("参数为空");
        }
        if (tenant.getTenantPhone() == null){
            throw new RuntimeException("手机号为空");
        }
        EduTenant eduTenant = super.getOneLink(new QueryWrapper<EduTenant>().eq("tenant_phone", tenant.getTenantPhone()));
        if (eduTenant != null&&eduTenant.getUserStatus()==0) {
            throw new RuntimeException("该手机号已注册，用户状态为禁用");
        }
        if (eduTenant != null&&eduTenant.getUserStatus()==1) {
            throw new RuntimeException("该手机号已注册，用户状态正常");
        }
        if (eduTenant == null){
            this.save(tenant);
        }
    }

    @Override
    @Transactional
    public void deleteByIds(List<Long> ids) {
        HashSet<Long> set = new HashSet<>(ids);
        if (set==null || set.isEmpty()) {
            throw new RuntimeException("参数为空");
        }
        List<EduTenant> tenants = this.listByIds(set);
        if (tenants == null || tenants.isEmpty()) {
            throw new RuntimeException("用户不存在或已注销");
        }
        List<EduTenantVo> eduTenantVos = fillStatus(tenants);
        ArrayList<Long> longs = new ArrayList<>();
        eduTenantVos.forEach(eduTenantVo -> {
            if (Objects.equals(eduTenantVo.getStatus(), UserStatus.USER_STATUS_DELETED.getCode())) {
                throw new RuntimeException("包含已删除用户");
            } else if (Objects.equals(eduTenantVo.getStatus(), UserStatus.USER_STATUS_DISABLED.getCode())) {
                throw new RuntimeException("包含已禁用用户");
            } else {
                longs.add(eduTenantVo.getId());
            }
            this.removeByIds(longs);
        });
    }

    @Override
    @Transactional
    public void updateTenant(EduTenant tenant) {
        if (tenant == null) throw new RuntimeException("参数为空");
        if (tenant.getId() == null) throw new RuntimeException("参数为空");
        EduTenant eduTenant = super.getOneLink(new QueryWrapper<EduTenant>().eq("id", tenant.getId()));
        if (eduTenant == null) throw new RuntimeException("用户不存在");
        if (eduTenant.getDeleteFlag() == 1)  throw new RuntimeException("用户已注销");
        if (eduTenant.getUserStatus() == 0) throw new RuntimeException("用户已禁用");
        if (tenant.getTenantPhone() != null){
            EduTenant eduTenant1 = super.getOneLink(new QueryWrapper<EduTenant>().eq("tenant_phone", tenant.getTenantPhone()));
            if (eduTenant1 != null) throw new RuntimeException("该手机号已注册");
        }
        this.updateById(tenant);
    }

    @Override
    public EduTenant getTenantById(Long id) {
        return super.getOneLink(new LambdaQueryWrapper<EduTenant>().eq(EduTenant::getId,id)
                .eq(EduTenant::getDeleteFlag,0)
                .eq(EduTenant::getUserStatus,1));
    }

    @Override
    public List<SysMenu> getMenuList(EduTenant principal) {

        if (principal ==null) throw new RuntimeException("用户未登录");
        EduTenant tenant = this.getTenantById(principal.getId());
        if (tenant == null) throw new RuntimeException("用户不存在");
        if (tenant.getUserStatus() == 0) throw new RuntimeException("用户已禁用");
        if (tenant.getDeleteFlag() == 1) throw new RuntimeException("用户已注销");
        if (tenant.getId() != null) {
            //用户角色列表
            List<Long> roleIdList = sysUserRoleService.getBaseMapper().selectList(new LambdaQueryWrapper<SysUserRole>()
                            .eq(SysUserRole::getId, tenant.getId())).stream()
                    .map(SysUserRole::getRoleId)
                    .collect(Collectors.toList());
            if (roleIdList.isEmpty()) return null;
            //菜单列表
            Set<Long> menuIdList = sysRoleMenuService.getBaseMapper().selectList(new LambdaQueryWrapper<SysRoleMenu>()
                            .in(SysRoleMenu::getRoleId, roleIdList)).stream()
                    .map(SysRoleMenu::getMenuId)
                    .collect(Collectors.toSet());
            //菜单列表内容
            List<SysMenu> list = sysMenuService.list(new LambdaQueryWrapper<SysMenu>()
                    .eq(SysMenu::getDeleteFlag, 0)
                    .eq(SysMenu::getStatus, 1)
                    .in(SysMenu::getId, menuIdList));
            return list;
        }
        return null;
    }

    @Override
    public List<SysRole> getRoleListByUserId(EduTenant principal) {
        if (principal ==null) throw new RuntimeException("用户未登录");
        if (principal.getId() == null)  throw new RuntimeException("用户不存在");
        //用户角色列表id
        List<Long> roleIdList = sysUserRoleService.getBaseMapper().selectList(new LambdaQueryWrapper<SysUserRole>()
                        .eq(SysUserRole::getId, principal.getId())).stream()
                .map(SysUserRole::getRoleId)
                .collect(Collectors.toList());
        if (roleIdList.isEmpty()) return null;
        //角色列表内容
        List<SysRole> list = sysRoleService.list(new LambdaQueryWrapper<SysRole>()
                .in(SysRole::getId, roleIdList)
                .eq(SysRole::getDeleteFlag, 0));
        return list;
    }

    @Override
    public void assignRolesToUser(AssignRolesOrMenus assignRolesOrMenus) {
        Assert.isTrue(ObjectUtil.isNotEmpty(assignRolesOrMenus),"请登录");
        Assert.isTrue(ObjectUtil.isNotEmpty(assignRolesOrMenus.getId()),"请选择需分配角色的用户");
        Assert.isTrue(ObjectUtil.isNotEmpty(assignRolesOrMenus.getIds()),"请选择角色");
        List<SysRole> roleList = sysRoleService.listByIds(assignRolesOrMenus.getIds());
        Assert.isTrue(ObjectUtil.isNotEmpty(roleList),"选择的角色不存在");
        Assert.isTrue(roleList.size() == assignRolesOrMenus.getIds().size(),"包含重复的角色或不存在的角色");
        //分配角色
        sysUserRoleService.saveOrUpdateBatch(assignRolesOrMenus.getIds().stream().map(roleId -> {
            SysUserRole sysUserRole = new SysUserRole();
            sysUserRole.setUserId(assignRolesOrMenus.getId());
            sysUserRole.setRoleId(roleId);
            return sysUserRole;
        }).collect(Collectors.toList()));
    }

    /**
     *
     * 填充用户状态
     */
    private List<EduTenantVo> fillStatus(List<EduTenant> list) {
        List<EduTenantVo> eduTenantVos = BeanUtil.copyToList(list, EduTenantVo.class);
        eduTenantVos.forEach(eduTenantVo -> {
            if (eduTenantVo.getDeleteFlag() == 1) {
                eduTenantVo.setStatus(UserStatus.USER_STATUS_DELETED.getCode());
            } else if (eduTenantVo.getUserStatus() == 0) {
                eduTenantVo.setStatus(UserStatus.USER_STATUS_DISABLED.getCode());
            } else {
                eduTenantVo.setStatus(UserStatus.USER_STATUS_NORMAL.getCode());
            }
        });
        return eduTenantVos;
    }

}
