package edu.travel.tenant.web;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.interfaces.InsertGroups;
import edu.travel.resp.BaseResponse;
import edu.travel.resp.PageResponse;
import edu.travel.tenant.dto.AssignRolesOrMenus;
import edu.travel.tenant.dto.EduTenantPageDto;
import edu.travel.tenant.dto.SysRolePageDto;
import edu.travel.tenant.entity.EduTenant;
import edu.travel.tenant.entity.SysRole;
import edu.travel.tenant.service.ISysRoleService;
import edu.travel.web.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * SysRoleController 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/2/6
 */

@RestController
@RequestMapping("/sysRole")
public class SysRoleController extends BaseController<SysRole> {

    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 新增管理角色
     */
    @PostMapping("/addRole")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> addRole( @RequestBody SysRole sysRole) {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        sysRoleService.add(sysRole,principal);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 删除角色多个
     */
    @PostMapping("/deleteRoleByIds")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> deleteRoleByIds(@RequestBody List<Long> ids) {
        HashSet<Long> longs = new HashSet<>(ids);
        sysRoleService.deleteByIds(longs);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 删除角色单个
     */
    @PostMapping("/deleteRoleById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> deleteRoleById(Long id) {
        sysRoleService.deleteById(id);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 更新角色
     */
    @PostMapping("/updateRoleById")
    public BaseResponse<Boolean> updateRoleById(@Validated(InsertGroups.class) @RequestBody SysRole sysRole) {
        sysRoleService.updateRoleById(sysRole);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 根据用户id查询用户角色
     */
    @GetMapping("/getRoleByUserId")
    public BaseResponse<List<SysRole>> getRoleByUserId(Long userId) {
        List<SysRole> sysRoles = sysRoleService.getRoleByUserId(userId);
        return new BaseResponse<>(200,"success",sysRoles) ;
    }

    /**
     * 为角色分配菜单
     * @param dto
     * @return
     */
    @PostMapping("/assignMenusToRole")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> assignMenusToRole(@RequestBody AssignRolesOrMenus dto) {
        sysRoleService.assignMenusToRole(dto);
        return new BaseResponse<>(200,"sucess",true);
    }

    @GetMapping("/getRolePageList")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<List<SysRole>> getRolePageList(SysRolePageDto sysRolePageDto) {
            IPage<SysRole> page = new Page<>(sysRolePageDto.getPageNum(), sysRolePageDto.getPageSize());
            IPage<SysRole> tenantIPage = sysRoleService.page(page, new LambdaQueryWrapper<SysRole>().eq(SysRole::getDeleteFlag, 0));
            return PageResponse.out(200, "success", tenantIPage.getRecords(), (int) tenantIPage.getTotal(), (int) tenantIPage.getSize());
        }
}
